"""
Module de gestion du formulaire de connexion pour FabriqueDoc.

Ce module contient les fonctions nécessaires pour gérer l'authentification
des utilisateurs dans l'application FabriqueDoc.
"""

# Importation des modules nécessaires
import streamlit as st  # Pour créer l'interface utilisateur web
import requests  # Pour effectuer des requêtes HTTP
from typing import Dict, Any  # Pour le typage des fonctions

def authentifier(username: str, password: str) -> Dict[str, Any]:
    """
    Authentifie l'utilisateur auprès du backend.

    Cette fonction envoie les identifiants de l'utilisateur au serveur
    et récupère un token d'accès si l'authentification réussit.

    Args:
        username (str): Nom d'utilisateur.
        password (str): Mot de passe.

    Returns:
        Dict[str, Any]: Réponse du serveur contenant le token d'accès si l'authentification réussit.
    """
    try:
        # Envoi d'une requête POST au serveur pour l'authentification
        response = requests.post(
            f"{st.session_state['fabriquedoc_endpoint']}/token/",
            data={"username": username, "password": password},
            timeout=10  # Limite le temps d'attente de la réponse à 10 secondes
        )
        response.raise_for_status()  # Lève une exception si la requête a échoué
        return response.json()  # Retourne la réponse du serveur en format JSON
    except requests.RequestException as e:
        # En cas d'erreur, affiche un message d'erreur et retourne un dictionnaire vide
        st.error(f"Erreur de connexion : {str(e)}")
        return {}

def mettre_a_jour_session(token: str) -> None:
    """
    Met à jour l'état de la session après une authentification réussie.

    Cette fonction stocke le token d'accès dans la session et marque l'utilisateur comme connecté.

    Args:
        token (str): Token d'accès reçu du serveur.
    """
    st.session_state['bearer_token'] = token  # Stocke le token dans la session
    st.session_state['logged_in'] = True  # Marque l'utilisateur comme connecté
    st.success("Connexion réussie!")  # Affiche un message de succès

def reinitialiser_session() -> None:
    """
    Réinitialise l'état de la session en cas d'échec d'authentification.

    Cette fonction efface le token d'accès et marque l'utilisateur comme déconnecté.
    """
    st.session_state['bearer_token'] = ""  # Efface le token de la session
    st.session_state['logged_in'] = False  # Marque l'utilisateur comme déconnecté
    st.error("Connexion échouée!")  # Affiche un message d'erreur

def login_form() -> None:
    """
    Affiche et gère le formulaire de connexion.

    Cette fonction crée un formulaire de connexion avec des champs pour le nom d'utilisateur
    et le mot de passe, et gère le processus d'authentification lorsque le formulaire est soumis.
    """
    with st.form(key='authentication'):
        # Création des champs de saisie pour le nom d'utilisateur et le mot de passe
        username = st.text_input("Nom d'utilisateur")
        password = st.text_input("Mot de passe", type="password")
        submit_button = st.form_submit_button(label='Se connecter')

        if submit_button:
            # Si le bouton de connexion est cliqué, tente d'authentifier l'utilisateur
            reponse = authentifier(username, password)
            if reponse and "access_token" in reponse:
                # Si l'authentification réussit, met à jour la session
                mettre_a_jour_session(reponse["access_token"])
            else:
                # Si l'authentification échoue, réinitialise la session
                reinitialiser_session()